@@ -1,19 +1,24 @@
-/* ************************************************************************ */
-/*																			*/
-/*  Tora - Neko Application Server											*/
-/*  Copyright (c)2008 Motion-Twin											*/
-/*																			*/
-/* This library is free software; you can redistribute it and/or			*/
-/* modify it under the terms of the GNU Lesser General Public				*/
-/* License as published by the Free Software Foundation; either				*/
-/* version 2.1 of the License, or (at your option) any later version.		*/
-/*																			*/
-/* This library is distributed in the hope that it will be useful,			*/
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of			*/
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU		*/
-/* Lesser General Public License or the LICENSE file for more details.		*/
-/*																			*/
-/* ************************************************************************ */
+/*
+ * Copyright (C)2005-2012 Haxe Foundation
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
+ */
 #include <httpd.h>
 #include <http_config.h>
 #include <http_core.h>
@@ -23,6 +28,7 @@
 #include "protocol.h"
 
 #ifndef OS_WINDOWS
+#	include <arpa/inet.h>
 #	define strcmpi	strcasecmp
 #endif
 
@@ -72,6 +78,7 @@ typedef struct {
 	int post_data_size;
 	bool headers_sent;
 	bool is_multipart;
+	bool is_form_post;
 	bool need_discard;
 } mcontext;
 
@@ -98,7 +105,7 @@ static void do_get_params( void *_c ) {
 	mcontext *c = (mcontext*)_c;
 	if( c->r->args )
 		protocol_send_raw_params(c->p,c->r->args);
-	if( c->post_data )
+	if( c->post_data && c->is_form_post )
 		protocol_send_raw_params(c->p,c->post_data);
 }
 
@@ -176,6 +183,7 @@ static int tora_handler( request_rec *r ) {
 	c->need_discard = false;
 	c->is_multipart = false;
 	c->headers_sent = false;
+	c->is_form_post = false;
 	c->r = r;
 	c->post_data = NULL;
 	c->xff = NULL;
@@ -207,6 +215,7 @@ static int tora_handler( request_rec *r ) {
 			}
 			c->post_data[tlen] = 0;
 			c->post_data_size = tlen;
+			c->is_form_post = ctype == NULL || (strstr(ctype,"urlencoded") != NULL);
 		}
 	}
 
