@@ -1,19 +1,24 @@
-/* ************************************************************************ */
-/*																			*/
-/*  Neko Standard Library													*/
-/*  Copyright (c)2005 Motion-Twin											*/
-/*																			*/
-/* This library is free software; you can redistribute it and/or			*/
-/* modify it under the terms of the GNU Lesser General Public				*/
-/* License as published by the Free Software Foundation; either				*/
-/* version 2.1 of the License, or (at your option) any later version.		*/
-/*																			*/
-/* This library is distributed in the hope that it will be useful,			*/
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of			*/
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU		*/
-/* Lesser General Public License or the LICENSE file for more details.		*/
-/*																			*/
-/* ************************************************************************ */
+/*
+ * Copyright (C)2005-2012 Haxe Foundation
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
+ */
 #include <neko_vm.h>
 
 /**
@@ -211,6 +216,148 @@ static value same_closure( value _f1, value _f2 ) {
 	return val_true;
 }
 
+// ------------- MERGE SORT HELPERS -----------------------------
+
+typedef struct {
+	value *arr;
+	value cmp;
+} m_sort;
+
+static int ms_compare( m_sort *m, int a, int b ) {
+	value v = val_call2(m->cmp,m->arr[a],m->arr[b]);
+	if( !val_is_int(v) ) return -1;
+	return val_int(v);
+}
+
+static void ms_swap( m_sort *m, int a, int b ) {
+	value tmp = m->arr[a];
+	m->arr[a] = m->arr[b];
+	m->arr[b] = tmp;
+}
+
+static int ms_lower( m_sort *m, int from, int to, int val ) {
+  	int len = to - from, half, mid;
+  	while( len > 0 ) {
+  		half = len>>1;
+		mid = from + half;
+		if( ms_compare(m, mid, val) < 0 ) {
+    		from = mid+1;
+    		len = len - half -1;
+   		} else
+   			len = half;
+  	}
+	return from;
+}
+
+static int ms_upper( m_sort *m, int from, int to, int val ) {
+	int len = to - from, half, mid;
+	while( len > 0 ) {
+		half = len>>1;
+		mid = from + half;
+		if( ms_compare(m, val, mid) < 0 )
+			len = half;
+		else {
+			from = mid+1;
+			len = len - half -1;
+		}
+	}
+	return from;
+}
+
+static int ms_gcd( int m, int n ) {
+ 	while( n != 0 ) {
+		int t = m % n;
+		m=n; n=t;
+	}
+ 	return m;
+ }
+
+
+static void ms_rotate( m_sort *m, int from, int mid, int to ) {
+	int n;
+	if( from==mid || mid==to ) return;
+	n = ms_gcd(to - from, mid - from);
+	while (n-- != 0) {
+		value val = m->arr[from+n];
+		int shift = mid - from;
+		int p1 = from+n, p2=from+n+shift;
+		while (p2 != from + n) {
+			m->arr[p1] = m->arr[p2];
+			p1=p2;
+			if( to - p2 > shift) p2 += shift;
+			else p2=from + (shift - (to - p2));
+		}
+		m->arr[p1] = val;
+	}
+}
+
+static void ms_do_merge( m_sort *m, int from, int pivot, int to, int len1, int len2 ) {
+	int first_cut, second_cut, len11, len22, new_mid;
+	if( len1 == 0 || len2==0 )
+		return;
+	if( len1+len2 == 2 ) {
+		if( ms_compare(m, pivot, from) < 0 )
+			ms_swap(m, pivot, from);
+   		return;
+  	}
+	if (len1 > len2) {
+		len11=len1>>1;
+		first_cut = from + len11;
+		second_cut = ms_lower(m, pivot, to, first_cut);
+		len22 = second_cut - pivot;
+	} else {
+		len22 = len2>>1;
+		second_cut = pivot + len22;
+		first_cut = ms_upper(m, from, pivot, second_cut);
+		len11=first_cut - from;
+	}
+	ms_rotate(m, first_cut, pivot, second_cut);
+	new_mid=first_cut+len22;
+	ms_do_merge(m, from, first_cut, new_mid, len11, len22);
+	ms_do_merge(m, new_mid, second_cut, to, len1 - len11, len2 - len22);
+}
+
+static void merge_sort_rec( m_sort *m, int from, int to ) {
+	int middle;
+	if( to - from < 12 ) {
+		// insert sort
+		int i;
+		if( to <= from ) return;
+		for(i=from+1;i<to;i++) {
+			int j = i;
+			while( j > from ) {
+				if( ms_compare(m,j,j-1) < 0 )
+					ms_swap(m,j-1,j);
+		    	else
+		    		break;
+			    j--;
+			}
+   		}
+		return;
+	}
+	middle = (from + to)>>1;
+	merge_sort_rec(m, from, middle);
+	merge_sort_rec(m, middle, to);
+	ms_do_merge(m, from, middle, to, middle-from, to - middle);
+}
+
+/**
+	merge_sort : array -> length:int -> cmp:function:2 -> void
+	<doc>
+	Sort the array using stable in-place merge sort and the [cmp] compare function.
+	</doc>
+**/
+static value merge_sort( value arr, value len, value cmp ) {
+	m_sort m;
+	val_check(arr,array);
+	val_check(len,int);
+	val_check_function(cmp,2);
+	m.arr = val_array_ptr(arr);
+	m.cmp = cmp;
+	merge_sort_rec(&m,0,val_int(len));
+	return val_null;
+}
+
 DEFINE_PRIM(float_bytes,2);
 DEFINE_PRIM(double_bytes,2);
 DEFINE_PRIM(float_of_bytes,2);
@@ -222,5 +369,6 @@ DEFINE_PRIM(test,0);
 DEFINE_PRIM(print_redirect,1);
 DEFINE_PRIM(set_trusted,1);
 DEFINE_PRIM(same_closure,2);
+DEFINE_PRIM(merge_sort,3);
 
 /* ************************************************************************ */
