@@ -1,19 +1,24 @@
-/* ************************************************************************ */
-/*																			*/
-/*  Neko Virtual Machine													*/
-/*  Copyright (c)2005 Motion-Twin											*/
-/*																			*/
-/* This library is free software; you can redistribute it and/or			*/
-/* modify it under the terms of the GNU Lesser General Public				*/
-/* License as published by the Free Software Foundation; either				*/
-/* version 2.1 of the License, or (at your option) any later version.		*/
-/*																			*/
-/* This library is distributed in the hope that it will be useful,			*/
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of			*/
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU		*/
-/* Lesser General Public License or the LICENSE file for more details.		*/
-/*																			*/
-/* ************************************************************************ */
+/*
+ * Copyright (C)2005-2012 Haxe Foundation
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
+ */
 #include <string.h>
 #include <stdio.h>
 #include "neko.h"
@@ -28,7 +33,11 @@
 #endif
 
 #ifdef NEKO_WINDOWS
+#ifdef NEKO_STANDALONE
+#	define GC_NOT_DLL
+#else
 #	define GC_DLL
+#endif
 #	define GC_WIN32_THREADS
 #endif
 
@@ -167,7 +176,7 @@ EXTERN value alloc_empty_string( unsigned int size ) {
 	if( size > max_string_size )
 		failure("max_string_size reached");
 	s = (vstring*)gc_alloc_private_big(size+sizeof(vstring));
-	s->t = VAL_STRING | (size << 3);
+	s->t = VAL_STRING | (size << TAG_BITS);
 	(&s->c)[size] = 0;
 	return (value)s;
 }
@@ -185,6 +194,13 @@ EXTERN value alloc_float( tfloat f ) {
 	return (value)v;
 }
 
+EXTERN value alloc_int32( int i ) {
+	vint32 *v = (vint32*)gc_alloc_private(sizeof(vint32));
+	v->t = VAL_INT32;
+	v->i = i;
+	return (value)v;
+}
+
 EXTERN value alloc_array( unsigned int n ) {
 	value v;
 	if( n == 0 )
@@ -192,7 +208,7 @@ EXTERN value alloc_array( unsigned int n ) {
 	if( n > max_array_size )
 		failure("max_array_size reached");
 	v = (value)gc_alloc_big(sizeof(varray)+(n - 1)*sizeof(value));
-	v->t = VAL_ARRAY | (n << 3);
+	v->t = VAL_ARRAY | (n << TAG_BITS);
 	return v;
 }
 
