@@ -1,19 +1,24 @@
-/* ************************************************************************ */
-/*																			*/
-/*  Neko Virtual Machine													*/
-/*  Copyright (c)2005 Motion-Twin											*/
-/*																			*/
-/* This library is free software; you can redistribute it and/or			*/
-/* modify it under the terms of the GNU Lesser General Public				*/
-/* License as published by the Free Software Foundation; either				*/
-/* version 2.1 of the License, or (at your option) any later version.		*/
-/*																			*/
-/* This library is distributed in the hope that it will be useful,			*/
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of			*/
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU		*/
-/* Lesser General Public License or the LICENSE file for more details.		*/
-/*																			*/
-/* ************************************************************************ */
+/*
+ * Copyright (C)2005-2012 Haxe Foundation
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
+ */
 #include <string.h>
 #include <stdlib.h>
 #include <math.h>
@@ -26,14 +31,10 @@
 #	define setjmp _setjmp
 #endif
 
-#ifdef NEKO_VCC
-	long _ftol( double f );
-	long _ftol2( double f) { return _ftol(f); };
-#endif
-
 extern value *neko_builtins;
 
 DEFINE_KIND(neko_k_kind);
+DEFINE_KIND(k_old_int32);
 
 /**
 	<doc>
@@ -610,7 +611,7 @@ static value builtin_varargs( value f ) {
 	<doc>Add two integers</doc>
 **/
 static value builtin_iadd( value a, value b ) {
-	return alloc_int( val_int(a) + val_int(b) );
+	return alloc_best_int( val_any_int(a) + val_any_int(b) );
 }
 
 /**
@@ -618,7 +619,7 @@ static value builtin_iadd( value a, value b ) {
 	<doc>Subtract two integers</doc>
 **/
 static value builtin_isub( value a, value b ) {
-	return alloc_int( val_int(a) - val_int(b) );
+	return alloc_best_int( val_any_int(a) - val_any_int(b) );
 }
 
 /**
@@ -626,7 +627,7 @@ static value builtin_isub( value a, value b ) {
 	<doc>Multiply two integers</doc>
 **/
 static value builtin_imult( value a, value b ) {
-	return alloc_int( val_int(a) * val_int(b) );
+	return alloc_best_int( val_any_int(a) * val_any_int(b) );
 }
 
 /**
@@ -634,9 +635,9 @@ static value builtin_imult( value a, value b ) {
 	<doc>Divide two integers. An error occurs if division by 0</doc>
 **/
 static value builtin_idiv( value a, value b ) {
-	if( b == (value)1 )
+	if( val_any_int(b) == 0 )
 		neko_error();
-	return alloc_int( val_int(a) / val_int(b) );
+	return alloc_best_int( val_any_int(a) / val_any_int(b) );
 }
 
 typedef union {
@@ -689,7 +690,7 @@ static value builtin_int( value f ) {
 	switch( val_type(f) ) {
 	case VAL_FLOAT:
 #ifdef	NEKO_WINDOWS
-		return alloc_int((int)val_float(f));
+		return alloc_best_int((int)val_float(f));
 #else
 		// in case of overflow, the result is unspecified by ISO
 		// so we have to make a module 2^32 before casting to int
@@ -710,14 +711,15 @@ static value builtin_int( value f ) {
 				else if( k >= 'a' && k <= 'f' )
 					h = (h << 4) | ((k - 'a') + 10);
 				else
-					return alloc_int(0);
+					return val_null;
 			}
-			return alloc_int(h);
+			return alloc_best_int(h);
 		}
 		h = strtol(c,&end,10);
-		return ( c == end ) ? val_null : alloc_int(h);
+		return ( c == end ) ? val_null : alloc_best_int(h);
 		}
 	case VAL_INT:
+	case VAL_INT32:
 		return f;
 	}
 	return val_null;
@@ -745,6 +747,8 @@ static value builtin_float( value f ) {
 	<doc>Returns the kind value of the abstract</doc>
 **/
 static value builtin_getkind( value v ) {
+	if( val_is_int32(v) )
+		return alloc_abstract(neko_k_kind,k_old_int32);
 	val_check(v,abstract);
 	return alloc_abstract(neko_k_kind,val_kind(v));
 }
@@ -755,7 +759,7 @@ static value builtin_getkind( value v ) {
 **/
 static value builtin_iskind( value v, value k ) {
 	val_check_kind(k,neko_k_kind);
-	return val_is_abstract(v) ? alloc_bool(val_kind(v) == (vkind)val_data(k)) : val_false;
+	return val_is_abstract(v) ? alloc_bool(val_kind(v) == (vkind)val_data(k)) : (val_data(k) == k_old_int32 ? alloc_bool(val_is_int32(v)) : val_false);
 }
 
 /** <doc><h2>Hashtable Builtins</h2></doc> **/
@@ -1092,7 +1096,7 @@ static value builtin_rethrow( value v ) {
 	<doc>Return true if [v] is not [false], not [null] and not 0</doc>
 **/
 static value builtin_istrue( value f ) {
-	return alloc_bool(f != val_false && f != val_null && f != alloc_int(0));
+	return alloc_bool(f != val_false && f != val_null && f != alloc_int(0) && (val_is_int(f) || val_tag(f) != VAL_INT32 || val_int32(f) != 0));
 }
 
 /**
@@ -1100,7 +1104,7 @@ static value builtin_istrue( value f ) {
 	<doc>Return true if [v] is [false] or [null] or [0]</doc>
 **/
 static value builtin_not( value f ) {
-	return alloc_bool(f == val_false || f == val_null || f == alloc_int(0));
+	return alloc_bool(f == val_false || f == val_null || f == alloc_int(0) || (!val_is_int(f) && val_tag(f) == VAL_INT32 && val_int32(f) == 0));
 }
 
 /**
@@ -1123,6 +1127,7 @@ static value builtin_not( value f ) {
 static value builtin_typeof( value v ) {
 	switch( val_type(v) ) {
 	case VAL_INT:
+	case VAL_INT32:
 		return alloc_int(1);
 	case VAL_NULL:
 		return alloc_int(0);
